@@ -1439,9 +1439,10 @@ bool backup_start(ds_ctxt *ds_data, ds_ctxt *ds_meta,
 
         if (!backup_files_from_datadir(ds_data, fil_path_to_mysql_datadir,
 	                               "aws-kms-key") ||
-            !backup_files_from_datadir(ds_data,
-                                       aria_log_dir_path,
-                                       "aria_log")) {
+            (aria_log_dir_path &&
+             !backup_files_from_datadir(ds_data,
+                                        aria_log_dir_path,
+                                        "aria_log"))) {
 		return false;
 	}
 
@@ -1893,8 +1894,6 @@ copy_back()
 		return(false);
 	}
 
-	srv_max_n_threads = 1000;
-
 	/* copy undo tablespaces */
 
 	Copy_back_dst_dir dst_dir_buf;
@@ -2172,8 +2171,6 @@ decrypt_decompress()
 	bool ret;
 	datadir_iter_t *it = NULL;
 
-	srv_max_n_threads = 1000;
-
 	/* cd to backup directory */
 	if (my_setwd(xtrabackup_target_dir, MYF(MY_WME)))
 	{
