@@ -1,6 +1,6 @@
 /* ext_kyber.c
  *
- * Copyright (C) 2006-2022 wolfSSL Inc.
+ * Copyright (C) 2006-2023 wolfSSL Inc.
  *
  * This file is part of wolfSSL.
  *
@@ -25,10 +25,7 @@
 
 #include <wolfssl/wolfcrypt/settings.h>
 #include <wolfssl/wolfcrypt/error-crypt.h>
-
-#if defined(WOLFSSL_KYBER_90S) && defined(HAVE_PQM4)
-#error "KYBER-90s is not supported when building PQM4"
-#endif
+#include <wolfssl/wolfcrypt/logging.h>
 
 #ifdef WOLFSSL_HAVE_KYBER
 #include <wolfssl/wolfcrypt/ext_kyber.h>
@@ -41,21 +38,24 @@
 #endif
 
 #if defined (HAVE_LIBOQS)
+
+#include <wolfssl/wolfcrypt/port/liboqs/liboqs.h>
+
 static const char* OQS_ID2name(int id) {
     switch (id) {
-#ifdef WOLFSSL_KYBER_90S
-        case KYBER_LEVEL1: return OQS_KEM_alg_kyber_512_90s;
-        case KYBER_LEVEL3: return OQS_KEM_alg_kyber_768_90s;
-        case KYBER_LEVEL5: return OQS_KEM_alg_kyber_1024_90s;
-#else
         case KYBER_LEVEL1: return OQS_KEM_alg_kyber_512;
         case KYBER_LEVEL3: return OQS_KEM_alg_kyber_768;
         case KYBER_LEVEL5: return OQS_KEM_alg_kyber_1024;
-#endif
         default:           break;
     }
     return NULL;
 }
+
+int ext_kyber_enabled(int id)
+{
+    const char * name = OQS_ID2name(id);
+    return OQS_KEM_alg_is_enabled(name);
+}
 #endif
 
 /******************************************************************************/
@@ -101,10 +101,15 @@ int wc_KyberKey_Init(int type, KyberKey* key, void* heap, int devId)
 
         /* Keep type for parameters. */
         key->type = type;
+
+#ifdef WOLF_CRYPTO_CB
+        key->devCtx = NULL;
+        key->devId = devId;
+#endif
     }
 
-    (void)devId;
     (void)heap;
+    (void)devId;
 
     return ret;
 }
@@ -307,15 +312,30 @@ int wc_KyberKey_SharedSecretSize(KyberKey* key, word32* len)
 int wc_KyberKey_MakeKey(KyberKey* key, WC_RNG* rng)
 {
     int ret = 0;
+#ifdef HAVE_LIBOQS
     const char* algName = NULL;
     OQS_KEM *kem = NULL;
-    (void)rng;
+#endif
 
     /* Validate parameter. */
     if (key == NULL) {
         return BAD_FUNC_ARG;
     }
 
+#ifdef WOLF_CRYPTO_CB
+    #ifndef WOLF_CRYPTO_CB_FIND
+    if (key->devId != INVALID_DEVID)
+    #endif
+    {
+        ret = wc_CryptoCb_MakePqcKemKey(rng, WC_PQC_KEM_TYPE_KYBER,
+                                        key->type, key);
+        if (ret != CRYPTOCB_UNAVAILABLE)
+            return ret;
+        /* fall-through when unavailable */
+        ret = 0;
+    }
+#endif
+
 #ifdef HAVE_LIBOQS
     if (ret == 0) {
         algName = OQS_ID2name(key->type);
@@ -324,24 +344,23 @@ int wc_KyberKey_MakeKey(KyberKey* key, WC_RNG* rng)
         }
     }
 
-    if (ret == 0) {
-        algName = OQS_ID2name(key->type);
-        if (algName == NULL) {
-            ret = BAD_FUNC_ARG;
-        }
-    }
     if (ret == 0) {
         kem = OQS_KEM_new(algName);
         if (kem == NULL) {
             ret = BAD_FUNC_ARG;
         }
     }
+    if (ret == 0) {
+        ret = wolfSSL_liboqsRngMutexLock(rng);
+    }
     if (ret == 0) {
         if (OQS_KEM_keypair(kem, key->pub, key->priv) !=
             OQS_SUCCESS) {
             ret = BAD_FUNC_ARG;
         }
     }
+    wolfSSL_liboqsRngMutexUnlock();
+    OQS_KEM_free(kem);
 #endif /* HAVE_LIBOQS */
 #ifdef HAVE_PQM4
     if (ret == 0) {
@@ -356,8 +375,6 @@ int wc_KyberKey_MakeKey(KyberKey* key, WC_RNG* rng)
         ForceZero(key, sizeof(*key));
     }
 
-    OQS_KEM_free(kem);
-
     return ret;
 }
 
@@ -397,8 +414,14 @@ int wc_KyberKey_Encapsulate(KyberKey* key, unsigned char* ct, unsigned char* ss,
     WC_RNG* rng)
 {
     int ret = 0;
+#ifdef WOLF_CRYPTO_CB
+    word32 ctlen = 0;
+#endif
+#ifdef HAVE_LIBOQS
     const char * algName = NULL;
     OQS_KEM *kem = NULL;
+#endif
+
     (void)rng;
 
     /* Validate parameters. */
@@ -406,6 +429,24 @@ int wc_KyberKey_Encapsulate(KyberKey* key, unsigned char* ct, unsigned char* ss,
         ret = BAD_FUNC_ARG;
     }
 
+#ifdef WOLF_CRYPTO_CB
+    if (ret == 0) {
+        ret = wc_KyberKey_CipherTextSize(key, &ctlen);
+    }
+    if ((ret == 0)
+    #ifndef WOLF_CRYPTO_CB_FIND
+        && (key->devId != INVALID_DEVID)
+    #endif
+    ) {
+        ret = wc_CryptoCb_PqcEncapsulate(ct, ctlen, ss, KYBER_SS_SZ, rng,
+                                         WC_PQC_KEM_TYPE_KYBER, key);
+        if (ret != CRYPTOCB_UNAVAILABLE)
+            return ret;
+        /* fall-through when unavailable */
+        ret = 0;
+    }
+#endif
+
 #ifdef HAVE_LIBOQS
     if (ret == 0) {
         algName = OQS_ID2name(key->type);
@@ -419,11 +460,16 @@ int wc_KyberKey_Encapsulate(KyberKey* key, unsigned char* ct, unsigned char* ss,
             ret = BAD_FUNC_ARG;
         }
     }
+    if (ret == 0) {
+        ret = wolfSSL_liboqsRngMutexLock(rng);
+    }
     if (ret == 0) {
         if (OQS_KEM_encaps(kem, ct, ss, key->pub) != OQS_SUCCESS) {
             ret = BAD_FUNC_ARG;
         }
     }
+    wolfSSL_liboqsRngMutexUnlock();
+    OQS_KEM_free(kem);
 #endif /* HAVE_LIBOQS */
 #ifdef HAVE_PQM4
     if (ret == 0) {
@@ -434,8 +480,6 @@ int wc_KyberKey_Encapsulate(KyberKey* key, unsigned char* ct, unsigned char* ss,
     }
 #endif /* HAVE_PQM4 */
 
-    OQS_KEM_free(kem);
-
     return ret;
 }
 
@@ -480,9 +524,11 @@ int wc_KyberKey_Decapsulate(KyberKey* key, unsigned char* ss,
     const unsigned char* ct, word32 len)
 {
     int ret = 0;
-    const char * algName = NULL;
     word32 ctlen = 0;
+#ifdef HAVE_LIBOQS
+    const char * algName = NULL;
     OQS_KEM *kem = NULL;
+#endif
 
     /* Validate parameters. */
     if ((key == NULL) || (ss == NULL) || (ct == NULL)) {
@@ -495,6 +541,21 @@ int wc_KyberKey_Decapsulate(KyberKey* key, unsigned char* ss,
         ret = BUFFER_E;
     }
 
+#ifdef WOLF_CRYPTO_CB
+    if ((ret == 0)
+    #ifndef WOLF_CRYPTO_CB_FIND
+        && (key->devId != INVALID_DEVID)
+    #endif
+    ) {
+        ret = wc_CryptoCb_PqcDecapsulate(ct, ctlen, ss, KYBER_SS_SZ,
+                                         WC_PQC_KEM_TYPE_KYBER, key);
+        if (ret != CRYPTOCB_UNAVAILABLE)
+            return ret;
+        /* fall-through when unavailable */
+        ret = 0;
+    }
+#endif
+
 #ifdef HAVE_LIBOQS
     if (ret == 0) {
         algName = OQS_ID2name(key->type);
@@ -513,18 +574,18 @@ int wc_KyberKey_Decapsulate(KyberKey* key, unsigned char* ss,
             ret = BAD_FUNC_ARG;
         }
     }
+
+    OQS_KEM_free(kem);
 #endif /* HAVE_LIBOQS */
 #ifdef HAVE_PQM4
     if (ret == 0) {
-        if (crypto_kem_enc(ss, ct, key->priv) != 0) {
+        if (crypto_kem_dec(ss, ct, key->priv) != 0) {
             WOLFSSL_MSG("PQM4 Decapsulation failure.");
             ret = BAD_FUNC_ARG;
         }
     }
 #endif /* HAVE_PQM4 */
 
-    OQS_KEM_free(kem);
-
     return ret;
 
 }
@@ -543,7 +604,7 @@ int wc_KyberKey_Decapsulate(KyberKey* key, unsigned char* ss,
  * @param  [in]       in   Buffer holding encoded key.
  * @param  [in]       len  Length of data in buffer.
  * @return  0 on success.
- * @return  BAD_FUNC_ARG when key ot in is NULL.
+ * @return  BAD_FUNC_ARG when key or in is NULL.
  * @return  NOT_COMPILED_IN when key type is not supported.
  * @return  BUFFER_E when len is not the correct size.
  */
