@@ -48,6 +48,9 @@
 #include <debug_sync.h>
 #include "proxy_protocol.h"
 
+PSI_memory_key key_memory_NET_buff;
+PSI_memory_key key_memory_NET_compress_packet;
+
 #ifdef EMBEDDED_LIBRARY
 #undef MYSQL_SERVER
 #undef MYSQL_CLIENT
@@ -71,7 +74,13 @@ static int inline EXTRA_DEBUG_fflush(...) { return 0; }
 #ifdef MYSQL_SERVER
 #include <sql_class.h>
 #include <sql_connect.h>
-#define MYSQL_SERVER_my_error my_error
+
+static void inline MYSQL_SERVER_my_error(uint error, myf flags)
+{
+  my_error(error,
+           flags | MYF(global_system_variables.log_warnings > 3 ? ME_ERROR_LOG : 0));
+}
+
 #else
 static void inline MYSQL_SERVER_my_error(...) {}
 #endif
@@ -89,7 +98,7 @@ static void inline MYSQL_SERVER_my_error(...) {}
   the client should have a bigger max_allowed_packet.
 */
 
-#if defined(__WIN__) || !defined(MYSQL_SERVER)
+#if defined(_WIN32) || !defined(MYSQL_SERVER)
   /* The following is because alarms doesn't work on windows. */
 #ifndef NO_ALARM
 #define NO_ALARM
@@ -139,6 +148,8 @@ my_bool my_net_init(NET *net, Vio *vio, void *thd, uint my_flags)
   DBUG_ENTER("my_net_init");
   DBUG_PRINT("enter", ("my_flags: %u", my_flags));
   net->vio = vio;
+  net->read_timeout= 0;
+  net->write_timeout= 0;
   my_net_local_init(net);			/* Set some limits */
 
   if (net_allocate_new_packet(net, thd, my_flags))
@@ -151,6 +162,8 @@ my_bool my_net_init(NET *net, Vio *vio, void *thd, uint my_flags)
   net->where_b = net->remain_in_buf=0;
   net->net_skip_rest_factor= 0;
   net->last_errno=0;
+  net->pkt_nr_can_be_reset= 0;
+  net->using_proxy_protocol= 0;
   net->thread_specific_malloc= MY_TEST(my_flags & MY_THREAD_SPECIFIC);
   net->thd= 0;
 #ifdef MYSQL_SERVER
@@ -162,7 +175,7 @@ my_bool my_net_init(NET *net, Vio *vio, void *thd, uint my_flags)
   {
     /* For perl DBI/DBD. */
     net->fd= vio_fd(vio);
-#if defined(MYSQL_SERVER) && !defined(__WIN__)
+#if defined(MYSQL_SERVER) && !defined(_WIN32)
     if (!(test_flags & TEST_BLOCKING))
     {
       my_bool old_mode;
@@ -174,13 +187,26 @@ my_bool my_net_init(NET *net, Vio *vio, void *thd, uint my_flags)
   DBUG_RETURN(0);
 }
 
+
+/**
+  Allocate and assign new net buffer
+
+  @note In case of error the old buffer left
+
+  @retval TRUE error
+  @retval FALSE success
+*/
+
 my_bool net_allocate_new_packet(NET *net, void *thd, uint my_flags)
 {
+  uchar *tmp;
   DBUG_ENTER("net_allocate_new_packet");
-  if (!(net->buff=(uchar*) my_malloc((size_t) net->max_packet+
-				     NET_HEADER_SIZE + COMP_HEADER_SIZE +1,
-				     MYF(MY_WME | my_flags))))
+  if (!(tmp= (uchar*) my_malloc(key_memory_NET_buff,
+                                (size_t) net->max_packet +
+				NET_HEADER_SIZE + COMP_HEADER_SIZE + 1,
+				MYF(MY_WME | my_flags))))
     DBUG_RETURN(1);
+  net->buff= tmp;
   net->buff_end=net->buff+net->max_packet;
   net->write_pos=net->read_pos = net->buff;
   DBUG_RETURN(0);
@@ -192,6 +218,7 @@ void net_end(NET *net)
   DBUG_ENTER("net_end");
   my_free(net->buff);
   net->buff=0;
+  net->using_proxy_protocol= 0;
   DBUG_VOID_RETURN;
 }
 
@@ -222,11 +249,11 @@ my_bool net_realloc(NET *net, size_t length)
     my_real_read() may actually read 4 bytes depending on build flags and
     platform.
   */
-  if (!(buff= (uchar*) my_realloc((char*) net->buff, pkt_length +
+  if (!(buff= (uchar*) my_realloc(key_memory_NET_buff,
+                                  (char*) net->buff, pkt_length +
                                   NET_HEADER_SIZE + COMP_HEADER_SIZE + 1,
-                                  MYF(MY_WME |
-                                      (net->thread_specific_malloc ?
-                                       MY_THREAD_SPECIFIC : 0)))))
+                                  MYF(MY_WME | (net->thread_specific_malloc
+                                                ?  MY_THREAD_SPECIFIC : 0)))))
   {
     /* @todo: 1 and 2 codes are identical. */
     net->error= 1;
@@ -273,7 +300,7 @@ static int net_data_is_ready(my_socket sd)
   struct timeval tv;
   int res;
 
-#ifndef __WIN__
+#ifndef _WIN32
   /* Windows uses an _array_ of 64 fd's as default, so it's safe */
   if (sd >= FD_SETSIZE)
     return -1;
@@ -408,7 +435,6 @@ my_bool net_flush(NET *net)
 my_bool my_net_write(NET *net, const uchar *packet, size_t len)
 {
   uchar buff[NET_HEADER_SIZE];
-  int rc;
 
   if (unlikely(!net->vio)) /* nowhere to write */
     return 0;
@@ -445,7 +471,7 @@ my_bool my_net_write(NET *net, const uchar *packet, size_t len)
 #ifndef DEBUG_DATA_PACKETS
   DBUG_DUMP("packet_header", buff, NET_HEADER_SIZE);
 #endif
-  rc= MY_TEST(net_write_buff(net, packet, len));
+  my_bool rc= MY_TEST(net_write_buff(net, packet, len));
   MYSQL_NET_WRITE_DONE(rc);
   return rc;
 }
@@ -486,7 +512,7 @@ net_write_command(NET *net,uchar command,
   size_t length=len+1+head_len;			/* 1 extra byte for command */
   uchar buff[NET_HEADER_SIZE+1];
   uint header_size=NET_HEADER_SIZE+1;
-  int rc;
+  my_bool rc;
   DBUG_ENTER("net_write_command");
   DBUG_PRINT("enter",("length: %lu", (ulong) len));
 
@@ -637,8 +663,20 @@ net_real_write(NET *net,const uchar *packet, size_t len)
   my_bool net_blocking = vio_is_blocking(net->vio);
   DBUG_ENTER("net_real_write");
 
-#if defined(MYSQL_SERVER) && defined(USE_QUERY_CACHE)
-  query_cache_insert(net->thd, (char*) packet, len, net->pkt_nr);
+#if defined(MYSQL_SERVER)
+  THD *thd= (THD *)net->thd;
+#if defined(USE_QUERY_CACHE)
+  query_cache_insert(thd, (char*) packet, len, net->pkt_nr);
+#endif
+  if (likely(thd))
+  {
+    /*
+      Wait until pending operations (currently it is engine
+      asynchronous group commit) are finished before replying
+      to the client, to keep durability promise.
+    */
+    thd->async_state.wait_for_pending_ops();
+  }
 #endif
 
   if (unlikely(net->error == 2))
@@ -651,11 +689,10 @@ net_real_write(NET *net,const uchar *packet, size_t len)
     size_t complen;
     uchar *b;
     uint header_length=NET_HEADER_SIZE+COMP_HEADER_SIZE;
-    if (!(b= (uchar*) my_malloc(len + NET_HEADER_SIZE +
-                                COMP_HEADER_SIZE + 1,
-                                MYF(MY_WME |
-                                    (net->thread_specific_malloc ?
-                                     MY_THREAD_SPECIFIC : 0)))))
+    if (!(b= (uchar*) my_malloc(key_memory_NET_compress_packet,
+                                len + NET_HEADER_SIZE + COMP_HEADER_SIZE + 1,
+                                MYF(MY_WME | (net->thread_specific_malloc
+                                              ? MY_THREAD_SPECIFIC : 0)))))
     {
       net->error= 2;
       net->last_errno= ER_OUT_OF_RESOURCES;
@@ -696,7 +733,7 @@ net_real_write(NET *net,const uchar *packet, size_t len)
     if ((long) (length= vio_write(net->vio,pos,(size_t) (end-pos))) <= 0)
     {
       my_bool interrupted = vio_should_retry(net->vio);
-#if !defined(__WIN__)
+#if !defined(_WIN32)
       if ((interrupted || length == 0) && !thr_alarm_in_use(&alarmed))
       {
         if (!thr_alarm(&alarmed, net->write_timeout, &alarm_buff))
@@ -719,7 +756,7 @@ net_real_write(NET *net,const uchar *packet, size_t len)
 	}
       }
       else
-#endif /* !defined(__WIN__) */
+#endif /* !defined(_WIN32) */
 	if (thr_alarm_in_use(&alarmed) && !thr_got_alarm(&alarmed) &&
 	    interrupted)
       {
@@ -737,14 +774,26 @@ net_real_write(NET *net,const uchar *packet, size_t len)
 #endif /* !defined(MYSQL_SERVER) */
       net->error= 2;				/* Close socket */
       net->last_errno= (interrupted ? ER_NET_WRITE_INTERRUPTED :
-                               ER_NET_ERROR_ON_WRITE);
+                        ER_NET_ERROR_ON_WRITE);
+#ifdef MYSQL_SERVER
+      if (global_system_variables.log_warnings > 3)
+      {
+        my_printf_error(net->last_errno,
+                        "Could not write packet: fd: %lld  state: %d  "
+                        "errno: %d  vio_errno: %d  length: %ld",
+                        MYF(ME_ERROR_LOG),
+                        (longlong) vio_fd(net->vio), (int) net->vio->state,
+                        vio_errno(net->vio), net->last_errno, (ulong) (end-pos));
+        break;
+      }
+#endif
       MYSQL_SERVER_my_error(net->last_errno, MYF(0));
       break;
     }
     pos+=length;
     update_statistics(thd_increment_bytes_sent(net->thd, length));
   }
-#ifndef __WIN__
+#ifndef _WIN32
  end:
 #endif
 #ifdef HAVE_COMPRESS
@@ -908,6 +957,7 @@ static handle_proxy_header_result handle_proxy_header(NET *net)
     return RETRY;
   /* Change peer address in THD and ACL structures.*/
   uint host_errors;
+  net->using_proxy_protocol= 1;
   return (handle_proxy_header_result)thd_set_peer_addr(thd,
                          &(peer_info.peer_addr), NULL, peer_info.port,
                          false, &host_errors);
@@ -991,7 +1041,7 @@ retry:
             goto end;
           }
 
-#if !defined(__WIN__) && defined(MYSQL_SERVER)
+#if !defined(_WIN32) && defined(MYSQL_SERVER)
 	  /*
 	    We got an error that there was no data on the socket. We now set up
 	    an alarm to not 'read forever', change the socket to the blocking
@@ -1023,30 +1073,46 @@ retry:
 	      continue;
 	    }
 	  }
-#endif /* (!defined(__WIN__) && defined(MYSQL_SERVER) */
+#endif /* (!defined(_WIN32) && defined(MYSQL_SERVER) */
 	  if (thr_alarm_in_use(&alarmed) && !thr_got_alarm(&alarmed) &&
 	      interrupted)
 	  {					/* Probably in MIT threads */
 	    if (retry_count++ < net->retry_count)
 	      continue;
-	    EXTRA_DEBUG_fprintf(stderr, "%s: read looped with error %d, aborting thread\n",
-		    my_progname,vio_errno(net->vio));
+	    EXTRA_DEBUG_fprintf(stderr, "%s: read looped with error %d on "
+                                "file %lld, aborting thread\n",
+                                my_progname, vio_errno(net->vio),
+                                (longlong) vio_fd(net->vio));
 	  }
 #ifndef MYSQL_SERVER
-	  if (length != 0 && vio_errno(net->vio) == SOCKET_EINTR)
+	  if (length != 0 && vio_should_retry(net->vio))
 	  {
 	    DBUG_PRINT("warning",("Interrupted read. Retrying..."));
 	    continue;
 	  }
 #endif
-	  DBUG_PRINT("error",("Couldn't read packet: remain: %u  errno: %d  length: %ld",
+	  DBUG_PRINT("error",("Could not read packet: remain: %u  errno: %d  length: %ld",
 			      remain, vio_errno(net->vio), (long) length));
 	  len= packet_error;
 	  net->error= 2;				/* Close socket */
           net->last_errno= (vio_was_timeout(net->vio) ?
-                                   ER_NET_READ_INTERRUPTED :
-                                   ER_NET_READ_ERROR);
-          MYSQL_SERVER_my_error(net->last_errno, MYF(0));
+                            ER_NET_READ_INTERRUPTED :
+                            ER_NET_READ_ERROR);
+#ifdef MYSQL_SERVER
+          if (global_system_variables.log_warnings > 3)
+          {
+            my_printf_error(net->last_errno,
+                            "Could not read packet: fd: %lld  state: %d  "
+                            "remain: %u  errno: %d  vio_errno: %d  "
+                            "length: %lld",
+                            MYF(ME_ERROR_LOG),
+                            (longlong) vio_fd(net->vio), (int) net->vio->state,
+                            remain, vio_errno(net->vio), net->last_errno,
+                            (longlong) length);
+          }
+          else
+            my_error(net->last_errno, MYF(0));
+#endif /* MYSQL_SERVER */
 	  goto end;
 	}
 	remain -= (uint32) length;
@@ -1066,19 +1132,31 @@ retry:
 #endif
 	if (net->buff[net->where_b + 3] != (uchar) net->pkt_nr)
         {
-#ifndef MYSQL_SERVER
-          if (net->buff[net->where_b + 3] == (uchar) (net->pkt_nr -1))
+          if (net->pkt_nr_can_be_reset)
           {
             /*
-              If the server was killed then the server may have missed the
-              last sent client packet and the packet numbering may be one off.
+              We are using a protocol like semi-sync where master and slave
+              sends packets in parallel.
+              Copy current one as it can be useful for debugging.
             */
-            DBUG_PRINT("warning", ("Found possible out of order packets"));
-            expect_error_packet= 1;
+            net->pkt_nr= net->buff[net->where_b + 3];
           }
           else
+          {
+#ifndef MYSQL_SERVER
+            if (net->buff[net->where_b + 3] == (uchar) (net->pkt_nr -1))
+            {
+              /*
+                If the server was killed then the server may have missed the
+                last sent client packet and the packet numbering may be one off.
+              */
+              DBUG_PRINT("warning", ("Found possible out of order packets"));
+              expect_error_packet= 1;
+            }
+            else
 #endif
-            goto packets_out_of_order;
+              goto packets_out_of_order;
+          }
         }
         net->compress_pkt_nr= ++net->pkt_nr;
 #ifdef HAVE_COMPRESS
@@ -1239,7 +1317,10 @@ ulong
 my_net_read_packet(NET *net, my_bool read_from_server)
 {
   ulong reallen = 0;
-  return my_net_read_packet_reallen(net, read_from_server, &reallen); 
+  ulong length;
+  DBUG_ENTER("my_net_read_packet");
+  length= my_net_read_packet_reallen(net, read_from_server, &reallen);
+  DBUG_RETURN(length);
 }
 
 
