@@ -23,7 +23,7 @@
 #include "mariadb.h"
 #include "mysqld.h"
 #include "sql_priv.h"
-#ifndef __WIN__
+#ifndef _WIN32
 #include <netdb.h>        // getservbyname, servent
 #endif
 #include "sql_audit.h"
@@ -798,7 +798,7 @@ bool thd_init_client_charset(THD *thd, uint cs_number)
     {
       /* Disallow non-supported parser character sets: UCS2, UTF16, UTF32 */
       my_error(ER_WRONG_VALUE_FOR_VAR, MYF(0), "character_set_client",
-               cs->csname);
+               cs->cs_name.str);
       return true;
     }
     thd->org_charset= cs;
@@ -1109,8 +1109,8 @@ static int check_connection(THD *thd)
 void setup_connection_thread_globals(THD *thd)
 {
   DBUG_EXECUTE_IF("CONNECT_wait", {
-    extern MYSQL_SOCKET unix_sock;
-    while (unix_sock.fd >= 0)
+    extern Dynamic_array<MYSQL_SOCKET> listen_sockets;
+    while (listen_sockets.size())
       my_sleep(1000);
   });
   thd->store_globals();
