@@ -700,10 +700,6 @@ int compare_window_funcs_by_window_specs(Item_window_func *win_func1,
 }
 
 
-#define  SORTORDER_CHANGE_FLAG    1
-#define  PARTITION_CHANGE_FLAG    2
-#define  FRAME_CHANGE_FLAG        4
-
 typedef int (*Item_window_func_cmp)(Item_window_func *f1,
                                     Item_window_func *f2,
                                     void *arg); 
@@ -733,15 +729,15 @@ void order_window_funcs_by_window_specs(List<Item_window_func> *win_func_list)
 
   List_iterator_fast<Item_window_func> it(*win_func_list);
   Item_window_func *prev= it++;
-  prev->marker= SORTORDER_CHANGE_FLAG |
-                PARTITION_CHANGE_FLAG |
-                FRAME_CHANGE_FLAG;
+  prev->marker= (MARKER_SORTORDER_CHANGE |
+                 MARKER_PARTITION_CHANGE |
+                 MARKER_FRAME_CHANGE);
   Item_window_func *curr;
   while ((curr= it++))
   {
     Window_spec *win_spec_prev= prev->window_spec;
     Window_spec *win_spec_curr= curr->window_spec;
-    curr->marker= 0;
+    curr->marker= MARKER_UNUSED;
     if (!(win_spec_prev->partition_list == win_spec_curr->partition_list &&
           win_spec_prev->order_list == win_spec_curr->order_list))
     {
@@ -755,17 +751,17 @@ void order_window_funcs_by_window_specs(List<Item_window_func> *win_func_list)
         cmp= compare_window_spec_joined_lists(win_spec_prev, win_spec_curr);
       if (!(CMP_LT_C <= cmp && cmp <= CMP_GT_C))
       {
-        curr->marker= SORTORDER_CHANGE_FLAG |
-                      PARTITION_CHANGE_FLAG |
-                      FRAME_CHANGE_FLAG;
+        curr->marker= (MARKER_SORTORDER_CHANGE |
+                       MARKER_PARTITION_CHANGE |
+                       MARKER_FRAME_CHANGE);
       }
       else if (win_spec_prev->partition_list != win_spec_curr->partition_list)
       {
-        curr->marker|= PARTITION_CHANGE_FLAG | FRAME_CHANGE_FLAG;
+        curr->marker|= MARKER_PARTITION_CHANGE | MARKER_FRAME_CHANGE;
       }
     }
     else if (win_spec_prev->window_frame != win_spec_curr->window_frame)
-      curr->marker|= FRAME_CHANGE_FLAG;
+      curr->marker|= MARKER_FRAME_CHANGE;
 
     prev= curr;
   }
@@ -840,10 +836,10 @@ public:
     {
       //DBUG_ASSERT(info->read_record == rr_from_tempfile);
       rownum= 0;
-      io_cache= (IO_CACHE*)my_malloc(sizeof(IO_CACHE), MYF(0));
+      io_cache= (IO_CACHE*)my_malloc(PSI_INSTRUMENT_ME, sizeof(IO_CACHE), MYF(0));
       init_slave_io_cache(info->io_cache, io_cache);
 
-      ref_buffer= (uchar*)my_malloc(ref_length, MYF(0));
+      ref_buffer= (uchar*)my_malloc(PSI_INSTRUMENT_ME, ref_length, MYF(0));
       ref_buffer_valid= false;
     }
   }
@@ -2559,7 +2555,7 @@ void add_special_frame_cursors(THD *thd, Cursor_manager *cursor_manager,
       cursor_manager->add_cursor(bottom_bound);
       cursor_manager->add_cursor(top_bound);
       cursor_manager->add_cursor(current_row_pos);
-      DBUG_ASSERT(item_sum->fixed);
+      DBUG_ASSERT(item_sum->fixed());
       bool negative_offset= item_sum->sum_func() == Item_sum::LAG_FUNC;
       fc= new Frame_positional_cursor(*current_row_pos,
                                       *top_bound, *bottom_bound,
@@ -2575,7 +2571,7 @@ void add_special_frame_cursors(THD *thd, Cursor_manager *cursor_manager,
       Frame_cursor *top_bound= get_frame_cursor(thd, spec, true);
       cursor_manager->add_cursor(bottom_bound);
       cursor_manager->add_cursor(top_bound);
-      DBUG_ASSERT(item_sum->fixed);
+      DBUG_ASSERT(item_sum->fixed());
       Item *offset_item= new (thd->mem_root) Item_int(thd, 0);
       offset_item->fix_fields(thd, &offset_item);
       fc= new Frame_positional_cursor(*top_bound,
@@ -2591,7 +2587,7 @@ void add_special_frame_cursors(THD *thd, Cursor_manager *cursor_manager,
       Frame_cursor *top_bound= get_frame_cursor(thd, spec, true);
       cursor_manager->add_cursor(bottom_bound);
       cursor_manager->add_cursor(top_bound);
-      DBUG_ASSERT(item_sum->fixed);
+      DBUG_ASSERT(item_sum->fixed());
       Item *offset_item= new (thd->mem_root) Item_int(thd, 0);
       offset_item->fix_fields(thd, &offset_item);
       fc= new Frame_positional_cursor(*bottom_bound,
@@ -2607,7 +2603,7 @@ void add_special_frame_cursors(THD *thd, Cursor_manager *cursor_manager,
       Frame_cursor *top_bound= get_frame_cursor(thd, spec, true);
       cursor_manager->add_cursor(bottom_bound);
       cursor_manager->add_cursor(top_bound);
-      DBUG_ASSERT(item_sum->fixed);
+      DBUG_ASSERT(item_sum->fixed());
       Item *int_item= new (thd->mem_root) Item_int(thd, 1);
       Item *offset_func= new (thd->mem_root)
                               Item_func_minus(thd, item_sum->get_arg(1),
@@ -2796,7 +2792,7 @@ bool save_window_function_values(List<Item_window_func>& window_functions,
   Item *func;
   for (; (func = *func_ptr) ; func_ptr++)
   {
-    if (func->with_window_func && func->type() != Item::WINDOW_FUNC_ITEM)
+    if (func->with_window_func() && func->type() != Item::WINDOW_FUNC_ITEM)
       func->save_in_result_field(true);
   }
 
@@ -2878,7 +2874,7 @@ bool compute_window_func(THD *thd,
 
   List_iterator_fast<Group_bound_tracker> iter_part_trackers(partition_trackers);
   ha_rows rownum= 0;
-  uchar *rowid_buf= (uchar*) my_malloc(tbl->file->ref_length, MYF(0));
+  uchar *rowid_buf= (uchar*) my_malloc(PSI_INSTRUMENT_ME, tbl->file->ref_length, MYF(0));
 
   while (true)
   {
@@ -2998,6 +2994,14 @@ bool Window_func_runner::add_function_to_run(Item_window_func *win_func)
       my_error(ER_NOT_SUPPORTED_YET, MYF(0),
                "COUNT(DISTINCT) aggregate as window function");
       return true;
+    case Item_sum::JSON_ARRAYAGG_FUNC:
+      my_error(ER_NOT_SUPPORTED_YET, MYF(0),
+               "JSON_ARRAYAGG() aggregate as window function");
+      return true;
+    case Item_sum::JSON_OBJECTAGG_FUNC:
+      my_error(ER_NOT_SUPPORTED_YET, MYF(0),
+               "JSON_OBJECTAGG() aggregate as window function");
+      return true;
     default:
       break;
   }
@@ -3018,7 +3022,8 @@ bool Window_func_runner::exec(THD *thd, TABLE *tbl, SORT_INFO *filesort_result)
     win_func->set_phase_to_computation();
     // TODO(cvicentiu) Setting the aggregator should probably be done during
     // setup of Window_funcs_sort.
-    win_func->window_func()->set_aggregator(Aggregator::SIMPLE_AGGREGATOR);
+    win_func->window_func()->set_aggregator(thd,
+                                            Aggregator::SIMPLE_AGGREGATOR);
   }
   it.rewind();
 
@@ -3091,11 +3096,11 @@ bool Window_funcs_sort::setup(THD *thd, SQL_SELECT *sel,
       return true;
     it++;
     win_func= it.peek();
-  } while (win_func && !(win_func->marker & SORTORDER_CHANGE_FLAG));
+  } while (win_func && !(win_func->marker & MARKER_SORTORDER_CHANGE));
 
   /*
     The sort criteria must be taken from the last win_func in the group of
-    adjacent win_funcs that do not have SORTORDER_CHANGE_FLAG. This is
+    adjacent win_funcs that do not have MARKER_SORTORDER_CHANGE. This is
     because the sort order must be the most specific sorting criteria defined
     within the window function group. This ensures that we sort the table
     in a way that the result is valid for all window functions belonging to
