@@ -30,7 +30,7 @@
 #include "wsrep_utils.h"
 #include "wsrep_xid.h"
 #include "wsrep_thd.h"
-#include "wsrep_mysqld.h"
+#include "wsrep_server_state.h"
 
 #include <cstdio>
 #include <cstdlib>
@@ -172,6 +172,36 @@ static void* wsrep_sst_joiner_monitor_thread(void *arg __attribute__((unused)))
   return NULL;
 }
 
+/* return true if character can be a part of a filename */
+static bool filename_char(int const c)
+{
+  return isalnum(c) || (c == '-') || (c == '_') || (c == '.');
+}
+
+/* return true if character can be a part of an address string */
+static bool address_char(int const c)
+{
+  return filename_char(c) ||
+         (c == ':') || (c == '[') || (c == ']') || (c == '/');
+}
+
+static bool check_request_str(const char* const str,
+                              bool (*check) (int c),
+                              bool log_warn = true)
+{
+  for (size_t i(0); str[i] != '\0'; ++i)
+  {
+    if (!check(str[i]))
+    {
+      if (log_warn) WSREP_WARN("Illegal character in state transfer request: %i (%c).",
+                               str[i], str[i]);
+      return true;
+    }
+  }
+
+  return false;
+}
+
 bool wsrep_sst_method_check (sys_var *self, THD* thd, set_var* var)
 {
   if ((! var->save_result.string_value.str) ||
@@ -183,6 +213,16 @@ bool wsrep_sst_method_check (sys_var *self, THD* thd, set_var* var)
     return 1;
   }
 
+  /* check also that method name is alphanumeric string  */
+  if (check_request_str(var->save_result.string_value.str,
+                        filename_char, false))
+  {
+    my_error(ER_WRONG_VALUE_FOR_VAR, MYF(0), var->var->name.str,
+             var->save_result.string_value.str ?
+             var->save_result.string_value.str : "NULL");
+    return 1;
+  }
+
   return 0;
 }
 
@@ -299,6 +339,15 @@ void wsrep_sst_auth_init ()
 
 bool  wsrep_sst_donor_check (sys_var *self, THD* thd, set_var* var)
 {
+  if ((! var->save_result.string_value.str) ||
+      (var->save_result.string_value.length > (FN_REFLEN -1))) // safety
+  {
+    my_error(ER_WRONG_VALUE_FOR_VAR, MYF(0), var->var->name.str,
+             var->save_result.string_value.str ?
+             var->save_result.string_value.str : "NULL");
+    return 1;
+  }
+
   return 0;
 }
 
@@ -336,9 +385,14 @@ static bool wsrep_sst_complete (THD*                thd,
   if ((state == Wsrep_server_state::s_joiner ||
        state == Wsrep_server_state::s_initialized))
   {
-    Wsrep_server_state::instance().sst_received(client_service,
-       rcode);
-    WSREP_INFO("SST succeeded for position %s", start_pos_buf);
+    if (Wsrep_server_state::instance().sst_received(client_service, rcode))
+    {
+      failed= true;
+    }
+    else
+    {
+      WSREP_INFO("SST succeeded for position %s", start_pos_buf);
+    }
   }
   else
   {
@@ -598,7 +652,9 @@ static void* sst_joiner_thread (void* a)
     else
     {
       // Read state ID (UUID:SEQNO) followed by wsrep_gtid_domain_id (if any).
+      unsigned long int domain_id= wsrep_gtid_domain_id;
       const char *pos= strchr(out, ' ');
+      WSREP_DEBUG("SST state ID tmp=%s out=%s pos=%p", tmp, out, pos);
 
       if (!pos) {
 
@@ -608,6 +664,13 @@ static void* sst_joiner_thread (void* a)
           WSREP_WARN("Did not find domain ID from SST script output '%s'. "
                      "Domain ID must be set manually to keep binlog consistent",
                      out);
+	  if (wsrep_gtid_domain_id)
+	  {
+	    WSREP_INFO("This node is configured to use wsrep_gtid_domain_id=%lu by user.",
+		       domain_id);
+	    wsrep_gtid_server.domain_id= (uint32)domain_id;
+	    wsrep_gtid_domain_id= (uint32)domain_id;
+	  }
         }
         err= sst_scan_uuid_seqno (out, &ret_uuid, &ret_seqno);
 
@@ -731,7 +794,7 @@ static int sst_append_env_var(wsp::env&   env,
   return -env.error();
 }
 
-#ifdef __WIN__
+#ifdef _WIN32
 /*
   Space, single quote, ampersand, backquote, I/O redirection
   characters, caret, all brackets, plus, exclamation and comma
@@ -805,7 +868,7 @@ static size_t estimate_cmd_len (bool* extra_args)
         else if (IS_REQ_ESCAPING(c))
         {
           cmd_len++;
-#ifdef __WIN__
+#ifdef _WIN32
           quotation= true;
 #endif
         }
@@ -834,7 +897,7 @@ static size_t estimate_cmd_len (bool* extra_args)
             else if (IS_REQ_ESCAPING(c))
             {
               cmd_len++;
-#ifdef __WIN__
+#ifdef _WIN32
               quotation= true;
 #endif
             }
@@ -890,7 +953,7 @@ static void copy_orig_argv (char* cmd_str)
         else if (IS_REQ_ESCAPING(c))
         {
           plain= false;
-#ifdef __WIN__
+#ifdef _WIN32
           quotation= true;
 #endif
         }
@@ -930,7 +993,7 @@ static void copy_orig_argv (char* cmd_str)
                 c = *arg++;
                 if (IS_REQ_ESCAPING(c))
                 {
-#ifdef __WIN__
+#ifdef _WIN32
                   *cmd_str++ = c;
 #else
                   *cmd_str++ = '\\';
@@ -970,7 +1033,7 @@ static void copy_orig_argv (char* cmd_str)
             else if (IS_REQ_ESCAPING(c))
             {
               plain= false;
-#ifdef __WIN__
+#ifdef _WIN32
               quotation= true;
 #endif
             }
@@ -1001,7 +1064,7 @@ static void copy_orig_argv (char* cmd_str)
           {
             if (IS_REQ_ESCAPING(c))
             {
-#ifdef __WIN__
+#ifdef _WIN32
               *cmd_str++ = c;
 #else
               *cmd_str++ = '\\';
@@ -1490,6 +1553,8 @@ static int sst_donate_mysqldump (const char*         addr,
                                         wsrep::seqno::undefined()));
   Wsrep_server_state::instance().sst_sent(sst_sent_gtid, ret);
 
+  wsrep_donor_monitor_end();
+
   return ret;
 }
 
@@ -1554,8 +1619,7 @@ static int run_sql_command(THD *thd, const char *query)
     return -1;
   }
 
-  mysql_parse(thd, thd->query(), thd->query_length(), &ps, FALSE, FALSE);
-
+  mysql_parse(thd, thd->query(), thd->query_length(), &ps);
   if (thd->is_error())
   {
     int const err= thd->get_stmt_da()->sql_errno();
@@ -1585,10 +1649,10 @@ static int sst_flush_tables(THD* thd)
   if (!is_supported_parser_charset(current_charset))
   {
       /* Do not use non-supported parser character sets */
-      WSREP_WARN("Current client character set is non-supported parser character set: %s", current_charset->csname);
+      WSREP_WARN("Current client character set is non-supported parser character set: %s", current_charset->cs_name.str);
       thd->variables.character_set_client= &my_charset_latin1;
       WSREP_WARN("For SST temporally setting character set to : %s",
-                 my_charset_latin1.csname);
+                 my_charset_latin1.cs_name.str);
   }
 
   if (run_sql_command(thd, "FLUSH TABLES WITH READ LOCK"))
@@ -1634,6 +1698,8 @@ static int sst_flush_tables(THD* thd)
     char content[100];
     snprintf(content, sizeof(content), "%s:%lld %d\n", wsrep_cluster_state_uuid,
              (long long)wsrep_locked_seqno, wsrep_gtid_server.domain_id);
+    WSREP_DEBUG("sst_flush_tables : %s:%lld %d", wsrep_cluster_state_uuid,
+                (long long)wsrep_locked_seqno, wsrep_gtid_server.domain_id);
     err= sst_create_file(flush_success, content);
 
     if (err)
@@ -1641,9 +1707,9 @@ static int sst_flush_tables(THD* thd)
 
     const char base_name[]= "tables_flushed";
     ssize_t const full_len= strlen(mysql_real_data_home) + strlen(base_name)+2;
-    char *real_name= (char*) malloc(full_len);
+    char *real_name= (char*) my_malloc(key_memory_WSREP, full_len, 0);
     sprintf(real_name, "%s/%s", mysql_real_data_home, base_name);
-    char *tmp_name= (char*) malloc(full_len + 4);
+    char *tmp_name= (char*) my_malloc(key_memory_WSREP, full_len + 4, 0);
     sprintf(tmp_name, "%s.tmp", real_name);
 
     FILE* file= fopen(tmp_name, "w+");
@@ -1671,8 +1737,8 @@ static int sst_flush_tables(THD* thd)
                      tmp_name, real_name, err,strerror(err));
       }
     }
-    free(real_name);
-    free(tmp_name);
+    my_free(real_name);
+    my_free(tmp_name);
     if (err)
       ha_disable_internal_writes(false);
   }
@@ -1739,6 +1805,7 @@ wait_signal:
 
           WSREP_INFO("Donor state reached");
 
+#ifdef ENABLED_DEBUG_SYNC
           DBUG_EXECUTE_IF("sync.wsrep_donor_state",
           {
             const char act[]=
@@ -1748,6 +1815,7 @@ wait_signal:
             assert(!debug_sync_set_action(thd.ptr,
                                           STRING_WITH_LEN(act)));
           };);
+#endif
 
           goto wait_signal;
         }
@@ -1907,35 +1975,6 @@ static int sst_donate_other (const char*        method,
   return arg.err;
 }
 
-/* return true if character can be a part of a filename */
-static bool filename_char(int const c)
-{
-  return isalnum(c) || (c == '-') || (c == '_') || (c == '.');
-}
-
-/* return true if character can be a part of an address string */
-static bool address_char(int const c)
-{
-  return filename_char(c) ||
-         (c == ':') || (c == '[') || (c == ']') || (c == '/');
-}
-
-static bool check_request_str(const char* const str,
-                              bool (*check) (int c))
-{
-  for (size_t i(0); str[i] != '\0'; ++i)
-  {
-    if (!check(str[i]))
-    {
-      WSREP_WARN("Illegal character in state transfer request: %i (%c).",
-                 str[i], str[i]);
-      return true;
-    }
-  }
-
-  return false;
-}
-
 int wsrep_sst_donate(const std::string& msg,
                      const wsrep::gtid& current_gtid,
                      const bool         bypass)
@@ -1943,7 +1982,7 @@ int wsrep_sst_donate(const std::string& msg,
   const char* method= msg.data();
   size_t method_len= strlen (method);
 
-  if (check_request_str(method, filename_char))
+  if (check_request_str(method, filename_char, true))
   {
     WSREP_ERROR("Bad SST method name. SST canceled.");
     return WSREP_CB_FAILURE;
@@ -1965,7 +2004,7 @@ int wsrep_sst_donate(const std::string& msg,
     addr= data;
   }
 
-  if (check_request_str(addr, address_char))
+  if (check_request_str(addr, address_char, true))
   {
     WSREP_ERROR("Bad SST address string. SST canceled.");
     return WSREP_CB_FAILURE;
