@@ -261,9 +261,7 @@ void mtr_t::rollback_to_savepoint(ulint begin, ulint end)
 /** Set create_lsn. */
 inline void fil_space_t::set_create_lsn(lsn_t lsn)
 {
-#ifndef SUX_LOCK_GENERIC
-  ut_ad(latch.is_write_locked());
-#endif
+  ut_ad(latch.have_wr());
   /* Concurrent log_checkpoint_low() must be impossible. */
   mysql_mutex_assert_owner(&log_sys.mutex);
   create_lsn= lsn;
@@ -635,6 +633,8 @@ static void log_write_low(const void *str, size_t size)
       len= trailer_offset - log_sys.buf_free % OS_FILE_LOG_BLOCK_SIZE;
     }
 
+    ut_ad(log_sys.is_physical());
+
     memcpy(log_sys.buf + log_sys.buf_free, str, len);
 
     size-= len;
@@ -985,7 +985,7 @@ void mtr_t::upgrade_buffer_fix(ulint savepoint, rw_lock_type_t rw_latch)
   ut_ad(slot.type == MTR_MEMO_BUF_FIX);
   buf_block_t *block= static_cast<buf_block_t*>(slot.object);
   ut_d(const auto state= block->page.state());
-  ut_ad(state > buf_page_t::UNFIXED);
+  ut_ad(state > buf_page_t::FREED);
   ut_ad(state > buf_page_t::WRITE_FIX || state < buf_page_t::READ_FIX);
   static_assert(int{MTR_MEMO_PAGE_S_FIX} == int{RW_S_LATCH}, "");
   static_assert(int{MTR_MEMO_PAGE_X_FIX} == int{RW_X_LATCH}, "");
