@@ -19,7 +19,7 @@
 */
 
 #include "fulltext.h"
-#if defined(__WIN__)
+#if defined(_WIN32)
 #include <fcntl.h>
 #else
 #include <stddef.h>
@@ -175,8 +175,7 @@ int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
 	maxbuffer_org= maxbuffer;
 	if (memavl < sizeof(BUFFPEK) * maxbuffer ||
 	    (keys= (memavl-sizeof(BUFFPEK)*maxbuffer)/
-             (sort_length+sizeof(char*))) <= 1 ||
-            keys < maxbuffer)
+             (sort_length+sizeof(char*))) <= 1)
 	{
 	  mi_check_print_error(info->sort_info->param,
 			       "myisam_sort_buffer_size is too small. Current myisam_sort_buffer_size: %llu  rows: %llu  sort_length: %u",
@@ -185,16 +184,26 @@ int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
           my_errno= ENOMEM;
 	  goto err;
 	}
+        if (keys < maxbuffer)
+        {
+          /*
+            There must be sufficient memory for at least one key per BUFFPEK,
+            otherwise repair by sort/parallel repair cannot operate.
+          */
+          keys= maxbuffer;
+          break;
+        }
       }
       while ((maxbuffer= (uint) (records/(keys-1)+1)) != maxbuffer_org);
     }
 
     if ((sort_keys= ((uchar **)
-                     my_malloc((size_t) (keys*(sort_length+sizeof(char*))+
+                     my_malloc(PSI_INSTRUMENT_ME,
+                               (size_t) (keys*(sort_length+sizeof(char*))+
                                          HA_FT_MAXBYTELEN), MYF(0)))))
     {
-      if (my_init_dynamic_array(&buffpek, sizeof(BUFFPEK), maxbuffer,
-                                MY_MIN(maxbuffer/2, 1000), MYF(0)))
+      if (my_init_dynamic_array(PSI_INSTRUMENT_ME, &buffpek, sizeof(BUFFPEK),
+                                maxbuffer, MY_MIN(maxbuffer/2, 1000), MYF(0)))
       {
 	my_free(sort_keys);
         sort_keys= 0;
@@ -227,6 +236,20 @@ int _create_index_by_sort(MI_SORT_PARAM *info,my_bool no_messages,
                               &tempfile,&tempfile_for_exceptions))
       == HA_POS_ERROR)
     goto err; /* purecov: tested */
+  if (maxbuffer >= keys)
+  {
+    /*
+      merge_many_buff will crash if maxbuffer >= keys as then we cannot store in memory
+      the keys for each buffer.
+    */
+    keys= maxbuffer + 1;
+    if (!(sort_keys= ((uchar **)
+                      my_realloc(PSI_INSTRUMENT_ME, sort_keys,
+                                 (size_t) (keys*(sort_length+sizeof(char*))+
+                                           HA_FT_MAXBYTELEN), MYF(MY_FREE_ON_ERROR)))))
+      goto err;
+  }
+
   if (maxbuffer == 0)
   {
     if (!no_messages)
@@ -406,12 +429,14 @@ static my_bool thr_find_all_keys_exec(MI_SORT_PARAM *sort_param)
       }
       while ((maxbuffer= (uint) (idx/(keys-1)+1)) != maxbuffer_org);
     }
-    if ((sort_keys= (uchar**) my_malloc((size_t)(keys * (sort_length + sizeof(char*)) +
+    if ((sort_keys= (uchar**) my_malloc(PSI_INSTRUMENT_ME,
+                    (size_t)(keys * (sort_length + sizeof(char*)) +
                    ((sort_param->keyinfo->flag & HA_FULLTEXT) ?
                     HA_FT_MAXBYTELEN : 0)), MYF(0))))
     {
-      if (my_init_dynamic_array(&sort_param->buffpek, sizeof(BUFFPEK),
-                                maxbuffer, MY_MIN(maxbuffer / 2, 1000), MYF(0)))
+      if (my_init_dynamic_array(PSI_INSTRUMENT_ME, &sort_param->buffpek,
+                                sizeof(BUFFPEK), maxbuffer,
+                                MY_MIN(maxbuffer / 2, 1000), MYF(0)))
       {
         my_free(sort_keys);
         sort_keys= NULL;          /* Safety against double free on error. */
@@ -612,7 +637,8 @@ int thr_write_keys(MI_SORT_PARAM *sort_param)
         length=param->sort_buffer_length;
         while (length >= MIN_SORT_BUFFER)
         {
-          if ((mergebuf= my_malloc((size_t) length, MYF(0))))
+          if ((mergebuf= my_malloc(PSI_INSTRUMENT_ME,
+                                   (size_t) length, MYF(0))))
               break;
           length=length*3/4;
         }
