@@ -266,7 +266,9 @@ int spider_udf_direct_sql_create_conn_key(
       + 1 + 1
       + direct_sql->tgt_default_file_length + 1
       + direct_sql->tgt_default_group_length + 1
-      + direct_sql->tgt_dsn_length;
+      + direct_sql->tgt_dsn_length + 1
+      + direct_sql->tgt_filedsn_length + 1
+      + direct_sql->tgt_driver_length;
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
   } else {
     direct_sql->conn_key_length
@@ -288,8 +290,14 @@ int spider_udf_direct_sql_create_conn_key(
     *direct_sql->conn_key = '0' + direct_sql->connection_channel;
   DBUG_PRINT("info",("spider tgt_wrapper=%s", direct_sql->tgt_wrapper));
   tmp_name = strmov(direct_sql->conn_key + 1, direct_sql->tgt_wrapper);
-  DBUG_PRINT("info",("spider tgt_host=%s", direct_sql->tgt_host));
-  tmp_name = strmov(tmp_name + 1, direct_sql->tgt_host);
+  if (direct_sql->tgt_host)
+  {
+    DBUG_PRINT("info",("spider tgt_host=%s", direct_sql->tgt_host));
+    tmp_name = strmov(tmp_name + 1, direct_sql->tgt_host);
+  } else {
+    DBUG_PRINT("info",("spider tgt_host=NULL"));
+    tmp_name++;
+  }
   my_sprintf(port_str, (port_str, "%05ld", direct_sql->tgt_port));
   DBUG_PRINT("info",("spider port_str=%s", port_str));
   tmp_name = strmov(tmp_name + 1, port_str);
@@ -380,6 +388,20 @@ int spider_udf_direct_sql_create_conn_key(
       tmp_name = strmov(tmp_name + 1, direct_sql->tgt_dsn);
     } else
       tmp_name++;
+    if (direct_sql->tgt_filedsn)
+    {
+      DBUG_PRINT("info",("spider tgt_filedsn=%s",
+        direct_sql->tgt_filedsn));
+      tmp_name = strmov(tmp_name + 1, direct_sql->tgt_filedsn);
+    } else
+      tmp_name++;
+    if (direct_sql->tgt_driver)
+    {
+      DBUG_PRINT("info",("spider tgt_driver=%s",
+        direct_sql->tgt_driver));
+      tmp_name = strmov(tmp_name + 1, direct_sql->tgt_driver);
+    } else
+      tmp_name++;
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
   }
 #endif
@@ -416,7 +438,7 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
   char *tmp_name, *tmp_host, *tmp_username, *tmp_password, *tmp_socket;
   char *tmp_wrapper, *tmp_db, *tmp_ssl_ca, *tmp_ssl_capath, *tmp_ssl_cert;
   char *tmp_ssl_cipher, *tmp_ssl_key, *tmp_default_file, *tmp_default_group;
-  char *tmp_dsn;
+  char *tmp_dsn, *tmp_filedsn, *tmp_driver;
   int *need_mon;
   bool tables_on_different_db_are_joinable = TRUE;
   DBUG_ENTER("spider_udf_direct_sql_create_conn");
@@ -457,6 +479,10 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
           (uint) (direct_sql->tgt_default_group_length + 1),
         &tmp_dsn,
           (uint) (direct_sql->tgt_dsn_length + 1),
+        &tmp_filedsn,
+          (uint) (direct_sql->tgt_filedsn_length + 1),
+        &tmp_driver,
+          (uint) (direct_sql->tgt_driver_length + 1),
         &need_mon, (uint) (sizeof(int)),
         NullS))
     ) {
@@ -535,6 +561,12 @@ SPIDER_CONN *spider_udf_direct_sql_create_conn(
     spider_maybe_memcpy_string(
       &conn->tgt_dsn, direct_sql->tgt_dsn, tmp_dsn,
       &conn->tgt_dsn_length, direct_sql->tgt_dsn_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_filedsn, direct_sql->tgt_filedsn, tmp_filedsn,
+      &conn->tgt_filedsn_length, direct_sql->tgt_filedsn_length);
+    spider_maybe_memcpy_string(
+      &conn->tgt_driver, direct_sql->tgt_driver, tmp_driver,
+      &conn->tgt_driver_length, direct_sql->tgt_driver_length);
     conn->tgt_ssl_vsc = direct_sql->tgt_ssl_vsc;
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
   } else {
@@ -1141,7 +1173,9 @@ int spider_udf_parse_direct_sql_param(
         SPIDER_PARAM_INT("cto", connect_timeout, 0);
         SPIDER_PARAM_STR("dff", tgt_default_file);
         SPIDER_PARAM_STR("dfg", tgt_default_group);
+        SPIDER_PARAM_STR("drv", tgt_driver);
         SPIDER_PARAM_STR("dsn", tgt_dsn);
+        SPIDER_PARAM_STR("fds", tgt_filedsn);
         SPIDER_PARAM_LONGLONG("prt", priority, 0);
         SPIDER_PARAM_INT("rto", net_read_timeout, 0);
         SPIDER_PARAM_STR("sca", tgt_ssl_ca);
@@ -1164,6 +1198,7 @@ int spider_udf_parse_direct_sql_param(
         error_num= parse.fail(true);
         goto error;
       case 6:
+        SPIDER_PARAM_STR("driver", tgt_driver);
         SPIDER_PARAM_STR("server", server_name);
         SPIDER_PARAM_STR("socket", tgt_socket);
         SPIDER_PARAM_HINT_WITH_MAX("iop", iop, 3, direct_sql->table_count, 0, 2);
@@ -1171,6 +1206,7 @@ int spider_udf_parse_direct_sql_param(
         error_num= parse.fail(true);
         goto error;
       case 7:
+        SPIDER_PARAM_STR("filedsn", tgt_filedsn);
         SPIDER_PARAM_STR("wrapper", tgt_wrapper);
         SPIDER_PARAM_STR("ssl_key", tgt_ssl_key);
         error_num= parse.fail(true);
@@ -1252,8 +1288,14 @@ int spider_udf_set_direct_sql_param_default(
 ) {
   bool check_socket;
   bool check_database;
+  bool check_default_file;
+  bool check_host;
+  bool check_port;
   bool socket_has_default_value;
   bool database_has_default_value;
+  bool default_file_has_default_value;
+  bool host_has_default_value;
+  bool port_has_default_value;
   int error_num, roop_count;
   DBUG_ENTER("spider_udf_set_direct_sql_param_default");
   if (direct_sql->server_name)
@@ -1279,10 +1321,35 @@ int spider_udf_set_direct_sql_param_default(
   } else {
     check_database = FALSE;
   }
-  if (check_socket || check_database)
+  if (
+    !direct_sql->tgt_default_file &&
+    direct_sql->tgt_default_group &&
+    (*spd_defaults_file || *spd_defaults_extra_file)
+  ) {
+    check_default_file = TRUE;
+  } else {
+    check_default_file = FALSE;
+  }
+  if (!direct_sql->tgt_host)
+  {
+    check_host = TRUE;
+  } else {
+    check_host = FALSE;
+  }
+  if (direct_sql->tgt_port == -1)
+  {
+    check_port = TRUE;
+  } else {
+    check_port = FALSE;
+  }
+  if (check_socket || check_database || check_default_file || check_host ||
+    check_port)
   {
     socket_has_default_value = check_socket;
     database_has_default_value = check_database;
+    default_file_has_default_value = check_default_file;
+    host_has_default_value = check_host;
+    port_has_default_value = check_port;
     if (direct_sql->tgt_wrapper)
     {
       for (roop_count = 0; roop_count < SPIDER_DBTON_SIZE; roop_count++)
@@ -1310,6 +1377,21 @@ int spider_udf_set_direct_sql_param_default(
               database_has_default_value = spider_dbton[roop_count].
                 db_util->database_has_default_value();
             }
+            if (check_default_file)
+            {
+              default_file_has_default_value = spider_dbton[roop_count].
+                db_util->default_file_has_default_value();
+            }
+            if (check_host)
+            {
+              host_has_default_value = spider_dbton[roop_count].
+                db_util->host_has_default_value();
+            }
+            if (check_port)
+            {
+              port_has_default_value = spider_dbton[roop_count].
+                db_util->port_has_default_value();
+            }
             break;
           }
         }
@@ -1318,6 +1400,9 @@ int spider_udf_set_direct_sql_param_default(
   } else {
     socket_has_default_value = FALSE;
     database_has_default_value = FALSE;
+    default_file_has_default_value = FALSE;
+    host_has_default_value = FALSE;
+    port_has_default_value = FALSE;
   }
 
   if (database_has_default_value)
@@ -1348,7 +1433,7 @@ int spider_udf_set_direct_sql_param_default(
     }
   }
 
-  if (!direct_sql->tgt_host)
+  if (host_has_default_value)
   {
     DBUG_PRINT("info",("spider create default tgt_host"));
     direct_sql->tgt_host_length = strlen(my_localhost);
@@ -1362,11 +1447,8 @@ int spider_udf_set_direct_sql_param_default(
     }
   }
 
-  if (
-    !direct_sql->tgt_default_file &&
-    direct_sql->tgt_default_group &&
-    (*spd_defaults_file || *spd_defaults_extra_file)
-  ) {
+  if (default_file_has_default_value)
+  {
     DBUG_PRINT("info",("spider create default tgt_default_file"));
     if (*spd_defaults_extra_file)
     {
@@ -1397,7 +1479,7 @@ int spider_udf_set_direct_sql_param_default(
     direct_sql->access_mode = 0;
 #endif
 
-  if (direct_sql->tgt_port == -1)
+  if (port_has_default_value)
   {
 #if defined(HS_HAS_SQLCOM) && defined(HAVE_HANDLERSOCKET)
     if (direct_sql->access_mode == 1)
@@ -1530,6 +1612,14 @@ void spider_udf_free_direct_sql_alloc(
   {
     spider_free(spider_current_trx, direct_sql->tgt_dsn, MYF(0));
   }
+  if (direct_sql->tgt_filedsn)
+  {
+    spider_free(spider_current_trx, direct_sql->tgt_filedsn, MYF(0));
+  }
+  if (direct_sql->tgt_driver)
+  {
+    spider_free(spider_current_trx, direct_sql->tgt_driver, MYF(0));
+  }
   if (direct_sql->conn_key)
   {
     spider_free(spider_current_trx, direct_sql->conn_key, MYF(0));
