@@ -20,8 +20,13 @@
 */
 
 static LEX_STRING spider_init_queries[] = {
+  /* Use the default SQL_MODE for this connection. */
   {C_STRING_WITH_LEN(
-    "SET @@SQL_MODE = REPLACE(@@SQL_MODE, 'ORACLE', '');"
+    "SET @@SQL_MODE = 'STRICT_TRANS_TABLES,ERROR_FOR_DIVISION_BY_ZERO,"
+                      "NO_AUTO_CREATE_USER,NO_ENGINE_SUBSTITUTION';"
+  )},
+  {C_STRING_WITH_LEN(
+    "SET @@OLD_MODE = CONCAT(@@OLD_MODE, ',UTF8_IS_UTF8MB3');"
   )},
   {C_STRING_WITH_LEN(
     "create table if not exists mysql.spider_xa("
