@@ -18,15 +18,10 @@
 #include <my_global.h>
 #include "mysql_version.h"
 #include "spd_environ.h"
-#if MYSQL_VERSION_ID < 50500
-#include "mysql_priv.h"
-#include <mysql/plugin.h>
-#else
 #include "sql_priv.h"
 #include "probes_mysql.h"
 #include "sql_class.h"
 #include "sql_analyse.h"
-#endif
 #include "spd_db_include.h"
 #include "spd_include.h"
 #include "spd_malloc.h"
